/**
 * @file lhaef_example.c
* Usage example for C wrapper for C++ HepML writing library
* HepML API
* LCG MCDB project, Monte Carlo Data Base
* http://mcdb.cern.ch
* 
* Sergey Belov <Sergey.Belov@cern.ch>, 2007
**/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lhaef.h"


void form_sample_article(void * doc)
{
    lhaef_general_description * sample;
    sample = calloc(1, sizeof(lhaef_general_description));
    strcpy(sample->title, "p,p->Wbbj->l,nu,b,b,j process from BestGenerator");
    strcpy(sample->abstract, " There are about 1.1 MEvents of BestGenerator p,p->Wbbj with leptonic decays of W. Spin correlations and contribution from the initial sea quarks were taken into account.");
    sample->experiment = calloc(1, sizeof(lhapdf_experiment) );
    strcpy(sample->experiment->experiment, "CMS");
    strcpy(sample->experiment->group, "RDMS");
    
    set_general_description(doc, sample);
    free(sample->experiment);
    free(sample);
    
    lhaef_file * f;
    f = calloc(1, sizeof(lhaef_file));
    f->n_events = 195644;
    f->size = 407736817;
    f->cross_section = 22.78;
    strcpy(f->location_path, "gsiftp://castorgrid.cern.ch/castor/cern.ch/sft/mcdb/storage/118/wbbj_mu.pev");
    
    add_file(doc, f);
    free(f);
    
    lhaef_generator * gen;
    gen = calloc(1, sizeof(lhaef_generator));
    strcpy(gen->name, "BestGenerator");
    strcpy(gen->version, "4.4p3");
    strcpy(gen->homepage, "http://BestGenerator.sinp.msu.ru");
    strcpy(gen->description, "Patched version of BestGenerator 4.4.3 to generate events in \"old\" stable format which is understandable by BestGenerator-Pythia interface (CPYTH 1.2.7)");
    set_generator(doc, gen);
    free(gen);
    
    
    lhaef_model * model;
    model = calloc(1, sizeof(lhaef_model) );
    strcpy(model->name, "SM, Feynman gauge");
    set_model(doc, model);
    free(model);
    
    lhaef_parameter * param;
    lhaef_math_notation * notation;
    param    = calloc(1, sizeof(lhaef_parameter) );
    notation = calloc(1, sizeof(lhaef_math_notation) );
    param->notation = notation;
    
    strcpy(param->name, "Ms");
    strcpy(param->value, "0.117");
    strcpy(notation->plain, "Ms");
    strcpy(notation->html, "m&lt;sub&gt;s&lt;/sub&gt;");
    add_model_parameter(doc, param);
    
    
    memset(param, 0, sizeof(lhaef_parameter));
    memset(notation, 0, sizeof(lhaef_math_notation));
    param->notation = notation;
    strcpy(param->name, "Mtau");
    strcpy(param->value, "1.77699");
    strcpy(notation->plain, "Mtau");
    strcpy(notation->html, "m&lt;sub&gt;&#964;&lt;/sub&gt;");
    add_model_parameter(doc, param);
    
    free(notation);
    free(param);
    
    lhaef_cut * cut;
    lhaef_math_notation * cut_obj_notation;
    cut = calloc(1, sizeof(lhaef_cut) );
    cut_obj_notation = calloc(1, sizeof(lhaef_math_notation) );
    cut->object_notation = cut_obj_notation;
    
    strcpy(cut->name, "object1");
    strcpy(cut->min_value, "min value 1");
    strcpy(cut->max_value, "max_value 1");
    strcpy(cut_obj_notation->plain, "obj1 plain notation" );
    add_cut(doc, cut, 1);
    
    memset(cut, 0, sizeof(lhaef_cut));
    memset(cut_obj_notation, 0, sizeof(lhaef_math_notation));
    cut->object_notation = cut_obj_notation;
    strcpy(cut->name, "object2");
    strcpy(cut->min_value, "min value 2");
    strcpy(cut->max_value, "max_value 2");
    strcpy(cut_obj_notation->html, "obj2 HTML notation" );
    add_cut(doc, cut, 2);
    
    memset(cut, 0, sizeof(lhaef_cut));
    memset(cut_obj_notation, 0, sizeof(lhaef_math_notation));
    cut->object_notation = cut_obj_notation;
    strcpy(cut->name, "object3");
    strcpy(cut->min_value, "min value 3");
    strcpy(cut->max_value, "max_value 3");
    strcpy(cut_obj_notation->latex, "obj3 latex notation" );
    add_cut(doc, cut, 2);
    
    free(cut_obj_notation);
    free(cut);
    
    
    
    lhaef_author * author;
    author = calloc(1, sizeof(lhaef_author));
    strcpy(author->firstName, "Lev");
    strcpy(author->lastName, "Doudko");
    strcpy(author->email, "Lev.Doudko[at]nospamcern.ch");
    strcpy(author->experiment, "CMS");
    strcpy(author->group, "CMS Top group");
    strcpy(author->organization, "SINP MSU");
    add_author(doc, author);
    free(author);
    
    
    lhaef_process * process;
    lhaef_math_notation * final_state;
    process     = calloc(1, sizeof(lhaef_process));
    final_state = calloc(1, sizeof(lhaef_math_notation));
    process->final_state_notation = final_state;
    
    strcpy(process->final_state, "Wbbj->l,nu,b,b,j");
    strcpy(final_state->plain, "Wbbj->l,nu,b,b,j");
    process->cross_section = 22.78;
    set_process(doc, process);
    free(final_state);
    free(process);
    
    
    lhaef_beam * beam;
    beam = calloc(1, sizeof(lhaef_beam));
    strcpy(beam->particle_name, "p");
    beam->energy = 7000;
    beam->energy_unit = GeV;
    set_process_beam(doc, 0, beam);
    set_process_beam(doc, 1, beam);
    free(beam);
    
    
    lhaef_pdf * pdf;
    pdf = calloc(1, sizeof(lhaef_pdf));
    strcpy(pdf->name, "CTEQ6L1");
    pdf->lhapdf_set = 3;
    set_process_beam_pdf(doc, 1, pdf);
    free(pdf);
    
    
    lhaef_alphas * alphas;
    alphas = calloc(1, sizeof(lhaef_alphas));
    alphas->nloops = 3;
    alphas->nflavours = 5;
    alphas->lambda = 2.0;
    set_process_beam_pdf_alphas(doc, 0, alphas);
    alphas->lambda = 3.0;
    alphas->nloops = 4;
    set_process_beam_pdf_alphas(doc, 1, alphas);
    alphas->nloops = 7;
    alphas->nflavours = 6;
    set_process_alphas(doc, alphas);
    free(alphas);
    
    
    lhaef_subprocess * subproc;
    subproc = calloc(1, sizeof(lhaef_subprocess));
    strcpy(subproc->name, "u,D -> nm,M,G,b,B");
    subproc->cross_section = 1.3221;
    subproc->cross_section_unit = pb;
    subproc->cross_section_upper_error = 2.68e-03;
    subproc->cross_section_lower_error = 2.68e-03;
    add_subprocess(doc, subproc, 2);
    memset(subproc->name, 0, sizeof(*subproc->name));
    strcpy(subproc->name, "(u,S -> nm,M,G,b,B)");
    subproc->cross_section = 5.2978e-02;
    subproc->cross_section_upper_error = 1.00e-04;
    subproc->cross_section_lower_error = 1.00e-04;
    add_subprocess(doc, subproc, 1);
    free(subproc);
    
}



int main()
{
    void * doc;
    char * hepml_doc;
    
    doc = init_lhaef_document();
    form_sample_article( doc );
    form_hepml_document( doc, &hepml_doc, 0, 1 );
    
    printf( "%s\n", hepml_doc );
    
    release_hepml_document( hepml_doc );
    release_lhaef_document( doc );
    
    return 0;
}
