//
// MCDB API: Article.cpp
// LCG MCDB project, Monte Carlo Data Base
// http://mcdb.cern.ch
// 
// Sergey Belov <Sergey.Belov@cern.ch>, 2007
//

#include "hepml.hpp"
#include "macro.hpp"

namespace mcdb {

using std::string;
using std::vector;    


Article::Article(): id_(0) { }
Article::~Article() { }

FUNC_GET(int&, Article, id)

int& Article::id(int id)
{
    return (id_ = id);
}

FUNC_SETGET(string&, Article, title)
FUNC_SETGET(string&, Article, abstract)
FUNC_SETGET(string&, Article, comments)
FUNC_SETGET(ExperimentGroup&, Article, experimentGroup)
FUNC_SETGET(vector<Author>&, Article, authors)
FUNC_GET(const string, Article, postDate)
FUNC_SETGET(Process&, Article, process)
FUNC_SETGET(Generator&, Article, generator)
FUNC_SETGET(Model&, Article, model)
FUNC_SETGET(CutsVector&, Article, cuts)
FUNC_SETGET(vector<File>&, Article, files)
FUNC_SETGET(vector<string>&, Article, relatedPapers)


Author::Author() { }
Author::~Author() { }

FUNC_SETGET(string&, Author, firstName)
FUNC_SETGET(string&, Author, lastName)
FUNC_SETGET(string&, Author, email)
FUNC_SETGET(string&, Author, experiment)
FUNC_SETGET(string&, Author, experimentGroup)
FUNC_SETGET(string&, Author, organization)


ExperimentGroup::ExperimentGroup(const string& exp, 
                                 const string& grp,
                                 const string& descr)
    : experiment(exp), group(grp), description(descr)
{
}


Model::Model() { }
Model::~Model() { }

FUNC_SETGET(string&,Model,name)
FUNC_SETGET(string&,Model,description)
FUNC_SETGET(Model::ParametersVector&,Model,parameters)

Model::Parameter::Parameter() { }

Model::Parameter::Parameter(const string& name, const string& value)
    : name_(name), value_(value)
{
}

Model::Parameter::~Parameter() { }

FUNC_SETGET(string&,Model::Parameter,name)
FUNC_SETGET(string&,Model::Parameter,value)
FUNC_SETGET(string&,Model::Parameter,description)
FUNC_SETGET(MathNotation&,Model::Parameter,mathNotation)

Generator::Generator() { }
Generator::~Generator() { }

FUNC_SETGET(string&, Generator, name)
FUNC_SETGET(string&, Generator, version)
FUNC_SETGET(string&, Generator, homepage)
FUNC_SETGET(string&, Generator, description)


Cut::Cut(): logic_(include_region), cutsetId_(0) { }
Cut::~Cut() { }

FUNC_SETGET(string&, Cut, object)
FUNC_SETGET(MathNotation&, Cut, objectNotation)
FUNC_SETGET(string&, Cut, minValue)
FUNC_SETGET(string&, Cut, maxValue)
FUNC_SETGET(Cut::Logic&, Cut, logic)
FUNC_GET(int&, Cut, cutsetId)

int& Cut::cutsetId(int id)
{
    return (cutsetId_ = id);
}


Process::Process() { }
Process::~Process() { }


FUNC_SETGET(Beam&, Process, beam1)
FUNC_SETGET(Beam&, Process, beam2)
FUNC_SETGET(string&, Process, finalState)
FUNC_SETGET(MathNotation&, Process, finalStateNotation)
FUNC_SETGET(QcdCoupling&, Process, qcdCoupling)
FUNC_SETGET(CrossSection&, Process, crossSection);
FUNC_SETGET(SubprocessVector&, Process, subprocesses)


Subprocess::Subprocess() : cutsetId_(0) { }
Subprocess::~Subprocess() { }

FUNC_SETGET(string&, Subprocess, notation)
FUNC_SETGET(CrossSection&, Subprocess, crossSection)
FUNC_SETGET(MathNotation&, Subprocess, factorisationScale)
FUNC_SETGET(MathNotation&, Subprocess, renormalisationScale)
FUNC_GET(int&, Subprocess, cutsetId)

int& Subprocess::cutsetId(int id)
{
    return (cutsetId_ = id);
}

string CrossSection::unitStr()
{
    return unitStr( unit );
}

string CrossSection::unitStr(Unit u)
{
    const string units[] = { "b", "mb", "mub", "nb", "pb", "fb", "ab" };
    
    return units[u];
}

string Beam::Energy::unitStr()
{
    return unitStr( unit );
}

string Beam::Energy::unitStr(Unit u)
{
    const string units[] = { "eV", "keV", "MeV", "GeV", "TeV" };
    
    return units[u];
}

} //namespace mcdb

