//
// MCDB API: File.cpp
// LCG MCDB project, Monte Carlo Data Base
// http://mcdb.cern.ch
// 
// Sergey Belov <Sergey.Belov@cern.ch>, 2007
//

#include "hepml.hpp"
#include "macro.hpp"

namespace mcdb {

using std::string;
using std::vector;

File::File() : id_(0), eventsNumber_(0), size_(0) { }

File::~File() { }

File& File::operator=(const File& f)
{
    eventsNumber_ = f.eventsNumber_;
    crossSection_ = f.crossSection_;
    size_ = f.size_;
    type_ = f.type_;
    id_   = f.id_;
    paths_    = f.paths_;
    checksum_ = f.checksum_;
    comments_ = f.comments_;
    
    return *this;
}



string File::typeStr() {
    return File::typeStr( type_ );
}

string File::typeStr(File::Type t)
{
    const string types[] = { "other", "data", "metadata", "generator-config", "generator-binary" };
    
    return types[t];
}

FUNC_SETGET(File::Type&, File, type)
FUNC_GET(int&, File, id)

int& File::id(int id)
{
    return (id_ = id);
}

FUNC_GET(unsigned long&, File, eventsNumber)

unsigned long& File::eventsNumber(unsigned long number)
{
    return (eventsNumber_ = number);
}
        
FUNC_GET(unsigned long&, File, size)
        
unsigned long& File::size(unsigned long size)
{
    return (size_ = size);
}
        
FUNC_SETGET(File::Checksum&, File, checksum)
FUNC_SETGET(CrossSection&, File, crossSection)
FUNC_SETGET(string&, File, comments)
FUNC_SETGET(vector<string>&, File, paths)


// Search
vector<string> File::findPaths(const string& substr)
{
    using namespace std;

    vector<string>::iterator p = paths_.begin();
    vector<string>::iterator e = paths_.end();
    vector<string> result;
    string to_find = substr;

//    if ( to_find.find("/") != string::npos)
//      to_find.insert( to_find.begin(), '/' );

    for (; p!=e; ++p) {
        if ( p->substr( p->size() - to_find.size(), to_find.size() ) == to_find ) {
            result.push_back( *p );
        }
    }

    return result;
}

bool File::findSubstrInName(const string& substr)
{
    using namespace std;
    vector<string>::iterator p = paths_.begin();
    vector<string>::iterator e = paths_.end();

    for (; p != e; ++p) {
        string name = "/" + getNameFromPath(*p);

        if ( name.find(substr) != string::npos ) {
            return true;
        }
    } // for

    return false;
}


bool File::findSubstrInPath(const string& substr)
{
    using namespace std;
    vector<string>::iterator p = paths_.begin();
    vector<string>::iterator e = paths_.end();

    for (; p != e; ++p) {
        if ( p->find(substr) != string::npos ) {
            return true;
        }
    } // for

    return false;
}


bool File::hasName(const string& name)
{
    vector<string>::iterator p = paths_.begin();
    vector<string>::iterator e = paths_.end();

    for (; p != e; ++p) {
        if ( name == getNameFromPath(*p) ) {
            return true;
        }
    } // for

    return false;
}

string File::getCastorPath()
{

    vector<string>::iterator p = paths_.begin();
    vector<string>::iterator e = paths_.end();

    for(; p !=e; ++p) {
        if ( p->find("rfio://") == 0 ) {
            return *p;
        }
    }

    return "";
}

string File::getNameFromPath(const string& path)
{
    using namespace std;

    size_t namePos = path.rfind("/");

    if ( namePos == string::npos ) {
        return path;
    }

    namePos++; // position of next symbol after slash
    if ( namePos >= path.size() ) { // end of path?
        return "";
    }

    return string( path, namePos );
}


bool File::isPlainName(const string& url)
{
    return ( url.find("/") == string::npos );
}


std::string File::Checksum::typeStr()
{
    return typeStr( type );
}

std::string File::Checksum::typeStr(File::Checksum::Type t)
{
    const string types[] = { "unknown", "mda5", "sha1", "crc32" };
    
    return types[t];
}

} //namespace mcdb

